@@ -74,8 +74,9 @@
 #include <setjmp.h>
 #include <assert.h>
 #include <uuid/uuid.h>
-#include "vector.h"
 #include "nilfs.h"
+#include "util.h"
+#include "vector.h"
 #include "nilfs_gc.h"
 #include "nilfs_cleaner.h"
 #include "cleaner_msg.h"
@@ -89,31 +90,13 @@
 #endif	/* SYSCONFDIR */
 #define NILFS_CLEANERD_CONFFILE	SYSCONFDIR "/nilfs_cleanerd.conf"
 
-#ifndef min_t
-#define min_t(type,x,y) \
-	({ type __x = (x); type __y = (y); __x < __y ? __x : __y; })
-#define max_t(type,x,y) \
-	({ type __x = (x); type __y = (y); __x > __y ? __x : __y; })
-#endif
-
-#define nilfs_cnt64_ge(a, b)	((__s64)(a) - (__s64)(b) >= 0)
-
-#ifndef ARRAY_SIZE
-#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof((arr)[0]))
-#endif
-
-#define timeval_to_timespec(tv, ts)		\
-do {						\
-	(ts)->tv_sec = (tv)->tv_sec;		\
-	(ts)->tv_nsec = (tv)->tv_usec * 1000;	\
-} while (0)
 
 #ifdef _GNU_SOURCE
 #include <getopt.h>
-const static struct option long_option[] = {
+static const struct option long_option[] = {
 	{"conffile", required_argument, NULL, 'c'},
 	{"help", no_argument, NULL, 'h'},
-	/* internal option for mount.nilfs2 only */
+	/* nofork option is obsolete. It does nothing even if passed */
 	{"nofork", no_argument, NULL, 'n'},
 	{"protection-period", required_argument, NULL, 'p'},
 	{"version", no_argument, NULL, 'V'},
@@ -141,10 +124,12 @@ const static struct option long_option[] = {
  * @running: running state
  * @fallback: fallback state
  * @retry_cleaning: retrying reclamation for protected segments
+ * @no_timeout: the next timeout will be 0 seconds
  * @ncleansegs: number of semgents cleaned per cycle
  * @cleaning_interval: cleaning interval
  * @target: target time for sleeping
  * @timeout: timeout value for sleeping
+ * @min_reclaimable_blocks: min. number of reclaimable blocks
  * @prev_nongc_ctime: previous nongc ctime
  * @recvq: receive queue
  * @recvq_name: receive queue name
@@ -158,6 +143,7 @@ const static struct option long_option[] = {
  * @mm_ncleansegs: number of segmetns cleaned per cycle (manual mode)
  * @mm_protection_period: protection period (manual mode)
  * @mm_cleaning_interval: cleaning interval (manual mode)
+ * @mm_min_reclaimable_blocks: min. number of reclaimable blocks (manual mode)
  */
 struct nilfs_cleanerd {
 	struct nilfs *nilfs;
@@ -167,11 +153,13 @@ struct nilfs_cleanerd {
 	int running;
 	int fallback;
 	int retry_cleaning;
+	int no_timeout;
 	int shutdown;
 	long ncleansegs;
 	struct timeval cleaning_interval;
 	struct timeval target;
 	struct timeval timeout;
+	unsigned long min_reclaimable_blocks;
 	__u64 prev_nongc_ctime;
 	mqd_t recvq;
 	char *recvq_name;
@@ -184,6 +172,7 @@ struct nilfs_cleanerd {
 	long mm_ncleansegs;
 	struct timeval mm_protection_period;
 	struct timeval mm_cleaning_interval;
+	unsigned long mm_min_reclaimable_blocks;
 };
 
 /**
@@ -246,6 +235,12 @@ static int nilfs_cleanerd_config(struct nilfs_cleanerd *cleanerd,
 	else
 		nilfs_opt_clear_mmap(cleanerd->nilfs);
 #endif	/* HAVE_MMAP */
+
+	if (cleanerd->config.cf_use_set_suinfo)
+		nilfs_opt_set_set_suinfo(cleanerd->nilfs);
+	else
+		nilfs_opt_clear_set_suinfo(cleanerd->nilfs);
+
 	nilfs_cleanerd_set_log_priority(cleanerd);
 
 	if (protection_period != ULONG_MAX) {
@@ -273,6 +268,8 @@ static int nilfs_cleanerd_reconfig(struct nilfs_cleanerd *cleanerd,
 	} else {
 		cleanerd->ncleansegs = config->cf_nsegments_per_clean;
 		cleanerd->cleaning_interval = config->cf_cleaning_interval;
+		cleanerd->min_reclaimable_blocks =
+				config->cf_min_reclaimable_blocks;
 		syslog(LOG_INFO, "configuration file reloaded");
 	}
 	return ret;
@@ -379,8 +376,8 @@ nilfs_cleanerd_create(const char *dev, const char *dir, const char *conffile)
 
 	cleanerd->cnoconv = nilfs_cnoconv_create(cleanerd->nilfs);
 	if (cleanerd->cnoconv == NULL) {
-		syslog(LOG_ERR, "failed to create checkpoint number converter "
-		       ": %m");
+		syslog(LOG_ERR,
+		       "failed to create checkpoint number converter: %m");
 		goto out_nilfs;
 	}
 
@@ -437,6 +434,12 @@ static int nilfs_cleanerd_automatic_suspend(struct nilfs_cleanerd *cleanerd)
 	return cleanerd->config.cf_min_clean_segments > 0;
 }
 
+static long nilfs_cleanerd_ncleansegs(struct nilfs_cleanerd *cleanerd)
+{
+	return cleanerd->running == 2 ?
+		cleanerd->mm_ncleansegs : cleanerd->ncleansegs;
+}
+
 static struct timeval *
 nilfs_cleanerd_cleaning_interval(struct nilfs_cleanerd *cleanerd)
 {
@@ -453,6 +456,14 @@ nilfs_cleanerd_protection_period(struct nilfs_cleanerd *cleanerd)
 		&cleanerd->config.cf_protection_period;
 }
 
+static unsigned long
+nilfs_cleanerd_min_reclaimable_blocks(struct nilfs_cleanerd *cleanerd)
+{
+	return cleanerd->running == 2 ?
+		cleanerd->mm_min_reclaimable_blocks :
+		cleanerd->min_reclaimable_blocks;
+}
+
 static void
 nilfs_cleanerd_reduce_ncleansegs_for_retry(struct nilfs_cleanerd *cleanerd)
 {
@@ -482,7 +493,7 @@ static int nilfs_segment_is_protected(struct nilfs *nilfs, __u64 segnum,
 		return -1;
 
 	segseq = nilfs_get_segment_seqnum(nilfs, segment, segnum);
-	if (nilfs_cnt64_ge(segseq, protseq))
+	if (cnt64_ge(segseq, protseq))
 		ret = 1;
 	nilfs_put_segment(nilfs, segment);
 	return ret;
@@ -516,7 +527,7 @@ static int nilfs_segments_still_reclaimable(struct nilfs *nilfs,
 }
 
 #ifndef FIFREEZE
-#define FIFREEZE 	_IOWR('X', 119, int)
+#define FIFREEZE	_IOWR('X', 119, int)
 #define FITHAW		_IOWR('X', 120, int)
 #endif
 
@@ -571,11 +582,12 @@ nilfs_cleanerd_select_segments(struct nilfs_cleanerd *cleanerd,
 	unsigned long long imp, thr;
 	int i;
 
-	nsegs = cleanerd->ncleansegs;
+	nsegs = nilfs_cleanerd_ncleansegs(cleanerd);
 	nilfs = cleanerd->nilfs;
 	config = &cleanerd->config;
 
-	if ((smv = nilfs_vector_create(sizeof(struct nilfs_segimp))) == NULL)
+	smv = nilfs_vector_create(sizeof(struct nilfs_segimp));
+	if (!smv)
 		return -1;
 
 	/* The segments that were more recently written to disk than
@@ -595,15 +607,19 @@ nilfs_cleanerd_select_segments(struct nilfs_cleanerd *cleanerd,
 		sustat->ss_nongc_ctime;
 
 	for (segnum = 0; segnum < sustat->ss_nsegs; segnum += n) {
-		count = (sustat->ss_nsegs - segnum < NILFS_CLEANERD_NSUINFO) ?
-			sustat->ss_nsegs - segnum : NILFS_CLEANERD_NSUINFO;
-		if ((n = nilfs_get_suinfo(nilfs, segnum, si, count)) < 0) {
+		count = min_t(__u64, sustat->ss_nsegs - segnum,
+			      NILFS_CLEANERD_NSUINFO);
+		n = nilfs_get_suinfo(nilfs, segnum, si, count);
+		if (n < 0) {
 			nssegs = n;
 			goto out;
 		}
 		for (i = 0; i < n; i++) {
-			if (nilfs_suinfo_reclaimable(&si[i]) &&
-			    ((imp = (*config->cf_selection_policy.p_importance)(&si[i])) < thr)) {
+			if (!nilfs_suinfo_reclaimable(&si[i]))
+				continue;
+
+			imp = config->cf_selection_policy.p_importance(&si[i]);
+			if (imp < thr) {
 				if (si[i].sui_lastmod < oldest)
 					oldest = si[i].sui_lastmod;
 				if (si[i].sui_lastmod < prottime) {
@@ -643,30 +659,74 @@ nilfs_cleanerd_select_segments(struct nilfs_cleanerd *cleanerd,
 	return nssegs;
 }
 
+static int oom_adjust(void)
+{
+	int fd, err;
+	const char *path, *score;
+	struct stat st;
+
+	/* Avoid oom-killer */
+	path = "/proc/self/oom_score_adj";
+	score = "-1000\n";
+
+	if (stat(path, &st)) {
+		/* oom_score_adj cannot be used, try oom_adj */
+		path = "/proc/self/oom_adj";
+		score = "-17\n";
+	}
+
+	fd = open(path, O_WRONLY);
+	if (fd < 0) {
+		fprintf(stderr, "can't adjust oom-killer's pardon %s, %m\n",
+			path);
+		return errno;
+	}
+
+	err = write(fd, score, strlen(score));
+	if (err < 0) {
+		fprintf(stderr, "can't adjust oom-killer's pardon %s, %m\n",
+			path);
+		close(fd);
+		return errno;
+	}
+	close(fd);
+	return 0;
+}
+
 #define DEVNULL	"/dev/null"
 #define ROOTDIR	"/"
 
-static int daemonize(int nochdir, int noclose, int nofork)
+static int daemonize(int nochdir, int noclose)
 {
 	pid_t pid;
 
-	if (!nofork) {
-		if ((pid = fork()) < 0)
-			return -1;
-		else if (pid != 0)
-			/* parent */
-			_exit(0);
-	}
+	pid = fork();
+	if (pid < 0)
+		return -1;
+	else if (pid != 0)
+		/* parent */
+		_exit(0);
 
-	/* child or nofork */
+	/* child */
 	if (setsid() < 0)
 		return -1;
 
 	/* umask(0); */
 
+	/* for ensuring I'm not a session leader */
+	pid = fork();
+	if (pid < 0)
+		return -1;
+	else if (pid != 0)
+		/* parent */
+		_exit(0);
+
 	if (!nochdir && (chdir(ROOTDIR) < 0))
 		return -1;
 
+	printf("NILFS_CLEANERD_PID=%lu\n", (unsigned long)getpid());
+	fflush(stdout);
+
 	if (!noclose) {
 		close(0);
 		close(1);
@@ -821,7 +881,7 @@ static int nilfs_cleanerd_recalc_interval(struct nilfs_cleanerd *cleanerd,
 	interval = nilfs_cleanerd_cleaning_interval(cleanerd);
 	/* timercmp() does not work for '>=' or '<='. */
 	/* curr >= target */
-	if (!timercmp(&curr, &cleanerd->target, <)) {
+	if (!timercmp(&curr, &cleanerd->target, <) || cleanerd->no_timeout) {
 		cleanerd->timeout.tv_sec = 0;
 		cleanerd->timeout.tv_usec = 0;
 		timeradd(&curr, interval, &cleanerd->target);
@@ -849,7 +909,7 @@ static int nilfs_cleanerd_respond(struct nilfs_cleanerd *cleanerd,
 			       "/nilfs-cleanerq-%s", uuidbuf);
 		if (ret < 0)
 			goto out;
-		
+
 		if (cleanerd->sendq >= 0)
 			mq_close(cleanerd->sendq);
 		ret = -1;
@@ -891,13 +951,13 @@ static int nilfs_cleanerd_cmd_getstat(struct nilfs_cleanerd *cleanerd,
 {
 	struct nilfs_cleaner_response res = {0};
 
-	if (cleanerd->running == 0) {
+	if (cleanerd->running == 0)
 		res.status = NILFS_CLEANER_STATUS_IDLE;
-	} else if (cleanerd->running > 0) {
+	else if (cleanerd->running > 0)
 		res.status = NILFS_CLEANER_STATUS_RUNNING;
-	} else {
+	else
 		res.status = NILFS_CLEANER_STATUS_SUSPENDED;
-	}
+
 	res.result = NILFS_CLEANER_RSP_ACK;
 	return nilfs_cleanerd_respond(cleanerd, req, &res);
 }
@@ -946,6 +1006,33 @@ static int nilfs_cleanerd_cmd_run(struct nilfs_cleanerd *cleanerd,
 		cleanerd->mm_cleaning_interval =
 			cleanerd->cleaning_interval;
 	}
+	/* minimal reclaimable blocks */
+	if (req2->args.valid & NILFS_CLEANER_ARG_MIN_RECLAIMABLE_BLOCKS) {
+		switch (req2->args.min_reclaimable_blocks_unit) {
+		case NILFS_CLEANER_ARG_UNIT_NONE:
+			if (req2->args.min_reclaimable_blocks >
+				nilfs_get_blocks_per_segment(cleanerd->nilfs))
+				goto error_inval;
+
+			cleanerd->mm_min_reclaimable_blocks =
+				req2->args.min_reclaimable_blocks;
+			break;
+		case NILFS_CLEANER_ARG_UNIT_PERCENT:
+			if (req2->args.min_reclaimable_blocks > 100)
+				goto error_inval;
+
+			cleanerd->mm_min_reclaimable_blocks =
+				(req2->args.min_reclaimable_blocks *
+				nilfs_get_blocks_per_segment(cleanerd->nilfs) +
+				99) / 100;
+			break;
+		default:
+			goto error_inval;
+		}
+	} else {
+		cleanerd->mm_min_reclaimable_blocks =
+			cleanerd->min_reclaimable_blocks;
+	}
 	/* number of passes */
 	if (req2->args.valid & NILFS_CLEANER_ARG_NPASSES) {
 		if (!req2->args.npasses)
@@ -1183,10 +1270,14 @@ static int nilfs_cleanerd_handle_clean_check(struct nilfs_cleanerd *cleanerd,
 		/* disk space is close to limit -- accelerate cleaning */
 		cleanerd->ncleansegs = config->cf_mc_nsegments_per_clean;
 		cleanerd->cleaning_interval = config->cf_mc_cleaning_interval;
+		cleanerd->min_reclaimable_blocks =
+				config->cf_mc_min_reclaimable_blocks;
 	} else {
 		/* continue to run */
 		cleanerd->ncleansegs = config->cf_nsegments_per_clean;
 		cleanerd->cleaning_interval = config->cf_cleaning_interval;
+		cleanerd->min_reclaimable_blocks =
+				config->cf_min_reclaimable_blocks;
 	}
 
 	return 0; /* do gc */
@@ -1238,9 +1329,9 @@ static int nilfs_cleanerd_handle_manual_mode(struct nilfs_cleanerd *cleanerd,
 			}
 		}
 	}
-	if (cleanerd->mm_nrestpasses == 0 && cleanerd->mm_nrestsegs == 0) {
+	if (cleanerd->mm_nrestpasses == 0 && cleanerd->mm_nrestsegs == 0)
 		nilfs_cleanerd_manual_end(cleanerd);
-	}
+
 	return 0;
 }
 
@@ -1278,36 +1369,80 @@ static void nilfs_cleanerd_progress(struct nilfs_cleanerd *cleanerd, int nsegs)
 {
 	if (cleanerd->running == 2) {
 		/* decrease remaining number of segments */
-		cleanerd->mm_nrestsegs = 
+		cleanerd->mm_nrestsegs =
 			max_t(long, cleanerd->mm_nrestsegs - nsegs, 0);
 	}
 }
 
-static ssize_t nilfs_cleanerd_clean_segments(struct nilfs_cleanerd *cleanerd,
-					     __u64 *segnums, size_t nsegs,
-					     __u64 protseq, __u64 prottime)
+static int nilfs_cleanerd_clean_segments(struct nilfs_cleanerd *cleanerd,
+					 __u64 *segnums, size_t nsegs,
+					 __u64 protseq, __u64 prottime,
+					 size_t *ndone)
 {
-	nilfs_cno_t protcno;
-	int ret, i;
+	struct nilfs_reclaim_params params;
+	struct nilfs_reclaim_stat stat;
+	int ret, i, sumsegs;
 
-	ret = nilfs_cnoconv_time2cno(cleanerd->cnoconv, prottime, &protcno);
+	params.flags = NILFS_RECLAIM_PARAM_PROTSEQ |
+		       NILFS_RECLAIM_PARAM_PROTCNO |
+		       NILFS_RECLAIM_PARAM_MIN_RECLAIMABLE_BLKS;
+	params.min_reclaimable_blks =
+			nilfs_cleanerd_min_reclaimable_blocks(cleanerd);
+	params.protseq = protseq;
+
+	ret = nilfs_cnoconv_time2cno(cleanerd->cnoconv, prottime,
+				     &params.protcno);
 	if (ret < 0) {
-		syslog(LOG_ERR, "cannot convert protection time to checkpoint "
+		syslog(LOG_ERR,
+		       "cannot convert protection time to checkpoint " \
 		       "number: %m");
 		goto out;
 	}
 
-	ret = nilfs_reclaim_segment(cleanerd->nilfs, segnums, nsegs,
-				    protseq, protcno);
-	if (ret > 0) {
-		for (i = 0; i < ret; i++)
+	memset(&stat, 0, sizeof(stat));
+	ret = nilfs_xreclaim_segment(cleanerd->nilfs, segnums, nsegs, 0,
+				     &params, &stat);
+	if (ret < 0) {
+		if (errno == ENOMEM) {
+			nilfs_cleanerd_reduce_ncleansegs_for_retry(cleanerd);
+			cleanerd->fallback = 1;
+			*ndone = 0;
+			ret = 0;
+		}
+		goto out;
+	}
+
+	*ndone = 0;
+
+	if (stat.cleaned_segs > 0) {
+		for (i = 0; i < stat.cleaned_segs; i++)
 			syslog(LOG_DEBUG, "segment %llu cleaned",
 			       (unsigned long long)segnums[i]);
-		nilfs_cleanerd_progress(cleanerd, ret);
+
+		nilfs_cleanerd_progress(cleanerd, stat.cleaned_segs);
 		cleanerd->fallback = 0;
 		cleanerd->retry_cleaning = 0;
 
-	} else if (ret == 0) {
+		*ndone += stat.cleaned_segs;
+	}
+
+	if (stat.deferred_segs > 0) {
+		sumsegs = stat.cleaned_segs + stat.deferred_segs;
+		for (i = stat.cleaned_segs; i < sumsegs; i++)
+			syslog(LOG_DEBUG, "segment %llu deferred",
+			       (unsigned long long)segnums[i]);
+
+		nilfs_cleanerd_progress(cleanerd, stat.deferred_segs);
+		cleanerd->fallback = 0;
+		cleanerd->retry_cleaning = 0;
+
+		if (!stat.cleaned_segs)
+			cleanerd->no_timeout = 1;
+
+		*ndone += stat.deferred_segs;
+	}
+
+	if (*ndone == 0) {
 		syslog(LOG_DEBUG, "no segments cleaned");
 
 		if (!cleanerd->retry_cleaning &&
@@ -1320,12 +1455,8 @@ static ssize_t nilfs_cleanerd_clean_segments(struct nilfs_cleanerd *cleanerd,
 		} else {
 			cleanerd->retry_cleaning = 0;
 		}
-
-	} else if (ret < 0 && errno == ENOMEM) {
-		nilfs_cleanerd_reduce_ncleansegs_for_retry(cleanerd);
-		cleanerd->fallback = 1;
-		ret = 0;
 	}
+
 out:
 	return ret;
 }
@@ -1340,7 +1471,8 @@ static int nilfs_cleanerd_clean_loop(struct nilfs_cleanerd *cleanerd)
 	__u64 prottime = 0, oldest = 0;
 	__u64 segnums[NILFS_CLDCONFIG_NSEGMENTS_PER_CLEAN_MAX];
 	sigset_t sigset;
-	int ns, ndone, ret;
+	size_t ndone;
+	int ns, ret;
 
 	sigemptyset(&sigset);
 	if (sigprocmask(SIG_SETMASK, &sigset, NULL) < 0) {
@@ -1372,12 +1504,16 @@ static int nilfs_cleanerd_clean_loop(struct nilfs_cleanerd *cleanerd)
 
 	cleanerd->ncleansegs = cleanerd->config.cf_nsegments_per_clean;
 	cleanerd->cleaning_interval = cleanerd->config.cf_cleaning_interval;
+	cleanerd->min_reclaimable_blocks =
+			cleanerd->config.cf_min_reclaimable_blocks;
 
 
 	if (nilfs_cleanerd_automatic_suspend(cleanerd))
 		nilfs_cleanerd_clean_check_pause(cleanerd);
 
 	while (!cleanerd->shutdown) {
+		cleanerd->no_timeout = 0;
+
 		if (sigprocmask(SIG_BLOCK, &sigset, NULL) < 0) {
 			syslog(LOG_ERR, "cannot set signal mask: %m");
 			return -1;
@@ -1411,10 +1547,10 @@ static int nilfs_cleanerd_clean_loop(struct nilfs_cleanerd *cleanerd)
 		       ns, (ns <= 1) ? "" : "s");
 		ndone = 0;
 		if (ns > 0) {
-			ndone = nilfs_cleanerd_clean_segments(
+			ret = nilfs_cleanerd_clean_segments(
 				cleanerd, segnums, ns, sustat.ss_prot_seq,
-				prottime);
-			if (ndone < 0)
+				prottime, &ndone);
+			if (ret < 0)
 				return -1;
 		} else {
 			cleanerd->retry_cleaning = 0;
@@ -1426,7 +1562,7 @@ static int nilfs_cleanerd_clean_loop(struct nilfs_cleanerd *cleanerd)
 		if (ret < 0)
 			return -1;
 
-	sleep:
+sleep:
 		if (sigprocmask(SIG_UNBLOCK, &sigset, NULL) < 0) {
 			syslog(LOG_ERR, "cannot set signal mask: %m");
 			return -1;
@@ -1445,7 +1581,7 @@ int main(int argc, char *argv[])
 	char canonical[PATH_MAX + 2];
 	const char *dev, *dir;
 	char *endptr;
-	int status, nofork, c;
+	int status, c;
 #ifdef _GNU_SOURCE
 	int option_index;
 #endif	/* _GNU_SOURCE */
@@ -1453,7 +1589,6 @@ int main(int argc, char *argv[])
 	progname = (strrchr(argv[0], '/') != NULL) ?
 		strrchr(argv[0], '/') + 1 : argv[0];
 	conffile = NILFS_CLEANERD_CONFFILE;
-	nofork = 0;
 	status = 0;
 	protection_period = ULONG_MAX;
 	dev = NULL;
@@ -1474,8 +1609,7 @@ int main(int argc, char *argv[])
 			nilfs_cleanerd_usage(progname);
 			exit(0);
 		case 'n':
-			/* internal option for mount.nilfs2 only */
-			nofork = 1;
+			/* ignore nofork option, do nothing */
 			break;
 		case 'p':
 			protection_period = strtoul(optarg, &endptr, 10);
@@ -1522,11 +1656,16 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	if (daemonize(0, 0, nofork) < 0) {
+	if (daemonize(0, 0) < 0) {
 		fprintf(stderr, "%s: %s\n", progname, strerror(errno));
 		exit(1);
 	}
 
+	if (oom_adjust() < 0) {
+		fprintf(stderr, "adjusting the OOM killer failed: %m\n");
+		exit(1);
+	}
+
 	openlog(progname, LOG_PID, LOG_DAEMON);
 	syslog(LOG_INFO, "start");
 
