@@ -16,7 +16,7 @@
  * systems, as well as non-32 bit systems.
  */
 
-int ext2fs_set_bit(int nr,void * addr)
+int ext2fs_set_bit(int nr, void *addr)
 {
 	int		mask, retval;
 	unsigned char	*ADDR = (unsigned char *) addr;
@@ -28,7 +28,7 @@ int ext2fs_set_bit(int nr,void * addr)
 	return retval;
 }
 
-int ext2fs_clear_bit(int nr, void * addr)
+int ext2fs_clear_bit(int nr, void *addr)
 {
 	int		mask, retval;
 	unsigned char	*ADDR = (unsigned char *) addr;
@@ -40,14 +40,14 @@ int ext2fs_clear_bit(int nr, void * addr)
 	return retval;
 }
 
-int ext2fs_test_bit(int nr, const void * addr)
+int ext2fs_test_bit(int nr, const void *addr)
 {
 	int			mask;
 	const unsigned char	*ADDR = (const unsigned char *) addr;
 
 	ADDR += nr >> 3;
 	mask = 1 << (nr & 0x07);
-	return (mask & *ADDR);
+	return mask & *ADDR;
 }
 
 #endif	/* !_EXT2_HAVE_ASM_BITOPS_ */
