@@ -36,8 +36,6 @@
 #include <stdlib.h>
 #endif	/* HAVE_STDLIB_H */
 
-#include <assert.h>
-
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif	/* HAVE_UNISTD_H */
@@ -77,6 +75,7 @@
 #endif	/* HAVE_BLKID_BLKID_H */
 
 #include "nilfs.h"
+#include "util.h"
 #include "nilfs_feature.h"
 #include "mkfs.h"
 #include "pathnames.h"
@@ -84,11 +83,6 @@
 
 typedef __u64  blocknr_t;
 
-#define BUG_ON(x)	   assert(!(x))
-#define ROUNDUP_DIV(n,m)   (((n) - 1) / (m) + 1)
-#define max_t(type,x,y) \
-	({ type __x = (x); type __y = (y); __x > __y ? __x : __y; })
-
 extern __u32 crc32_le(__u32 seed, unsigned char const *data, size_t length);
 #define nilfs_crc32(seed, data, length)  crc32_le(seed, data, length)
 
@@ -105,24 +99,21 @@ static const char badblocks[] = "/sbin/" BADBLOCKS_NAME;
 /*
  * Command interface primitives
  */
-extern char *optarg;
-extern int optind;
-
 char *progname = "mkfs.nilfs2";
 
 /* Options */
-static int quiet = 0;
-static int cflag = 0;
-static int nflag = 0;
-static int verbose = 0;
+static int quiet;
+static int cflag;
+static int nflag;
+static int verbose;
 static int discard = 1;
-static int force_overwrite = 0;
+static int force_overwrite;
 static unsigned long blocksize = NILFS_DEF_BLOCKSIZE;
 static unsigned long blocks_per_segment = NILFS_DEF_BLKS_PER_SEG;
 static unsigned long r_segments_percentage = NILFS_DEF_RESERVED_SEGMENTS;
 
-static time_t creation_time = 0;
-static char volume_label[80] = {0};
+static time_t creation_time;
+static char volume_label[80];
 static __u64 compat_array[NILFS_MAX_FEATURE_TYPES] = {
 	/* Compat */
 	0,
@@ -214,7 +205,7 @@ struct nilfs_disk_info {
 
 struct nilfs_segment_ref {
 	__u64 seq;                      /* Sequence number of a full segment */
-	blocknr_t start;                /* Start block of the partial segment 
+	blocknr_t start;                /* Start block of the partial segment
 					   having a valid super root */
 	blocknr_t free_blocks_count;
 	__u64 cno;                /* checkpoint number */
@@ -225,8 +216,8 @@ static void init_disk_layout(struct nilfs_disk_info *di, int fd,
 
 static inline blocknr_t count_free_blocks(struct nilfs_disk_info *di)
 {
-	return (di->blocks_per_segment *
-		(di->nsegments - di->nsegments_to_write));
+	return di->blocks_per_segment *
+		(di->nsegments - di->nsegments_to_write);
 }
 
 static inline blocknr_t
@@ -239,7 +230,7 @@ segment_start_blocknr(struct nilfs_disk_info *di, unsigned long segnum)
 /*
  * I/O primitives
  */
-static void **disk_buffer = NULL;
+static void **disk_buffer;
 static unsigned long disk_buffer_size;
 
 static void init_disk_buffer(long max_blocks);
@@ -304,11 +295,11 @@ static void too_small_segment(unsigned long, unsigned long);
 #ifdef __linux__
 
 #ifndef BLKDISCARD
-#define BLKDISCARD	_IO(0x12,119)
+#define BLKDISCARD	_IO(0x12, 119)
 #endif
 
 #ifndef BLKDISCARDZEROES
-#define BLKDISCARDZEROES _IO(0x12,124)
+#define BLKDISCARDZEROES _IO(0x12, 124)
 #endif
 
 /**
@@ -352,8 +343,12 @@ static int nilfs_mkfs_discard_zeroes_data(int fd)
 
 static void disk_scan(const char *device);
 static void check_mount(int fd, const char *device);
-static void check_safety_of_device_overwrite(int fd, const char *device);
 
+#if HAVE_LIBBLKID
+static void check_safety_of_device_overwrite(int fd, const char *device);
+#else
+#define check_safety_of_device_overwrite(fd, device) do {} while (0)
+#endif /* HAVE_LIBBLKID */
 
 /*
  * Routines to decide disk layout
@@ -364,7 +359,7 @@ static unsigned count_blockgrouped_file_blocks(unsigned entry_size,
 	unsigned long entries_per_block = blocksize / entry_size;
 
 	return group_desc_blocks_per_group + bitmap_blocks_per_group +
-		ROUNDUP_DIV(nr_initial_entries, entries_per_block);
+		DIV_ROUND_UP(nr_initial_entries, entries_per_block);
 }
 
 static unsigned count_ifile_blocks(void)
@@ -384,7 +379,7 @@ static unsigned count_sufile_blocks(void)
 {
 	unsigned long sufile_segment_usages_per_block
 		= blocksize / sizeof(struct nilfs_segment_usage);
-	return ROUNDUP_DIV(nr_initial_segments +
+	return DIV_ROUND_UP(nr_initial_segments +
 			   NILFS_SUFILE_FIRST_SEGMENT_USAGE_OFFSET,
 			   sufile_segment_usages_per_block);
 }
@@ -394,7 +389,7 @@ static unsigned count_cpfile_blocks(void)
 	const unsigned nr_initial_checkpoints = 1;
 	unsigned long cpfile_checkpoints_per_block
 		= blocksize / sizeof(struct nilfs_checkpoint);
-	return ROUNDUP_DIV(nr_initial_checkpoints +
+	return DIV_ROUND_UP(nr_initial_checkpoints +
 			   NILFS_CPFILE_FIRST_CHECKPOINT_OFFSET
 			   - 1 /* checkpoint number begins from 1 */,
 			   cpfile_checkpoints_per_block);
@@ -413,16 +408,15 @@ static unsigned count_dat_blocks(unsigned nr_dat_entries)
 	return nblocks;
 }
 
-static void nilfs_check_ondisk_sizes(void)
+static __attribute__((used)) void nilfs_check_ondisk_sizes(void)
 {
-	if (sizeof(struct nilfs_inode) > NILFS_MIN_BLOCKSIZE ||
-	    sizeof(struct nilfs_sufile_header) > NILFS_MIN_BLOCKSIZE ||
-	    sizeof(struct nilfs_segment_usage) > NILFS_MIN_BLOCKSIZE ||
-	    sizeof(struct nilfs_cpfile_header) > NILFS_MIN_BLOCKSIZE ||
-	    sizeof(struct nilfs_checkpoint) > NILFS_MIN_BLOCKSIZE ||
-	    sizeof(struct nilfs_dat_entry) > NILFS_MIN_BLOCKSIZE ||
-	    sizeof(struct nilfs_super_root) > NILFS_MIN_BLOCKSIZE)
-		perr("Internal error: too large on-disk structure");
+	BUILD_BUG_ON(sizeof(struct nilfs_inode) > NILFS_MIN_BLOCKSIZE);
+	BUILD_BUG_ON(sizeof(struct nilfs_sufile_header) > NILFS_MIN_BLOCKSIZE);
+	BUILD_BUG_ON(sizeof(struct nilfs_segment_usage) > NILFS_MIN_BLOCKSIZE);
+	BUILD_BUG_ON(sizeof(struct nilfs_cpfile_header) > NILFS_MIN_BLOCKSIZE);
+	BUILD_BUG_ON(sizeof(struct nilfs_checkpoint) > NILFS_MIN_BLOCKSIZE);
+	BUILD_BUG_ON(sizeof(struct nilfs_dat_entry) > NILFS_MIN_BLOCKSIZE);
+	BUILD_BUG_ON(sizeof(struct nilfs_super_root) > NILFS_MIN_BLOCKSIZE);
 }
 
 static unsigned long
@@ -437,7 +431,7 @@ __increment_segsum_size(unsigned long offset, unsigned item_size,
 		offset2 = offset + item_size * count;
 	else {
 		unsigned nitems_per_block = blocksize / item_size;
-		
+
 		count -= rest_items_in_block;
 		offset2 = blocksize *
 			(offset / blocksize + 1 + count / nitems_per_block) +
@@ -449,7 +443,7 @@ __increment_segsum_size(unsigned long offset, unsigned item_size,
 static void increment_segsum_size(struct nilfs_segment_info *si,
 				  unsigned nblocks_in_file, int dat_flag)
 {
-	unsigned binfo_size = dat_flag ? 
+	unsigned binfo_size = dat_flag ?
 		sizeof(__le64) /* offset */ : sizeof(struct nilfs_binfo_v);
 
 	si->sumbytes = __increment_segsum_size(si->sumbytes,
@@ -461,15 +455,16 @@ static void increment_segsum_size(struct nilfs_segment_info *si,
 static inline int my_log2(long i)
 {
 	int n;
-	
-	for (n = 0; i > 1; i >>= 1) n++;
+
+	for (n = 0; i > 1; i >>= 1)
+		n++;
 	return n;
 }
 
 static unsigned long nilfs_min_nsegments(struct nilfs_disk_info *di, long rp)
 {
 	/* Minimum number of full segments */
-	return max_t(unsigned long, 
+	return max_t(unsigned long,
 		     (rp * di->nsegments - 1) / 100 + 1, NILFS_MIN_NRSVSEGS) +
 		max_t(unsigned long, nr_initial_segments, NILFS_MIN_NUSERSEGS);
 }
@@ -502,7 +497,7 @@ static void init_disk_layout(struct nilfs_disk_info *di, int fd,
 
 	di->blocks_per_segment = blocks_per_segment;
 	segment_size = di->blocks_per_segment * blocksize;
-	first_segblk = ROUNDUP_DIV(NILFS_DISKHDR_SIZE, blocksize);
+	first_segblk = DIV_ROUND_UP(NILFS_DISKHDR_SIZE, blocksize);
 	di->first_segment_block = first_segblk;
 	if (first_segblk + NILFS_PSEG_MIN_BLOCKS > di->blocks_per_segment)
 		too_small_segment(di->blocks_per_segment,
@@ -518,8 +513,6 @@ static void init_disk_layout(struct nilfs_disk_info *di, int fd,
 		     "or shorten segments with -B option.", dev_size,
 		     (unsigned long long)segment_size * min_nsegments);
 	di->nseginfo = 0;
-
-	nilfs_check_ondisk_sizes();
 }
 
 static struct nilfs_segment_info *new_segment(struct nilfs_disk_info *di)
@@ -532,7 +525,7 @@ static struct nilfs_segment_info *new_segment(struct nilfs_disk_info *di)
 	memset(si, 0, sizeof(*si));
 
 	si->sumbytes = sizeof(struct nilfs_segment_summary);
-	si->nblk_sum = ROUNDUP_DIV(si->sumbytes, blocksize);
+	si->nblk_sum = DIV_ROUND_UP(si->sumbytes, blocksize);
 	si->start = di->first_segment_block; /* for segment 0 */
 	return si;
 }
@@ -567,7 +560,7 @@ static void fix_disk_layout(struct nilfs_disk_info *di)
 		if (di->nblocks_to_write < si->start + si->nblocks)
 			di->nblocks_to_write = si->start + si->nblocks;
 	}
-	di->nsegments_to_write = ROUNDUP_DIV(di->nblocks_to_write,
+	di->nsegments_to_write = DIV_ROUND_UP(di->nblocks_to_write,
 					     di->blocks_per_segment);
 }
 
@@ -591,7 +584,7 @@ static void add_file(struct nilfs_segment_info *si, ino_t ino,
 		increment_segsum_size(si, nblocks, dat_flag);
 		if (!dat_flag)
 			si->nvblocknrs += nblocks;
-		si->nblk_sum = ROUNDUP_DIV(si->sumbytes, blocksize);
+		si->nblk_sum = DIV_ROUND_UP(si->sumbytes, blocksize);
 	}
 }
 
@@ -614,7 +607,9 @@ int main(int argc, char *argv[])
 
 	if (cflag)
 		disk_scan(device);  /* check the block device */
-	if ((fd = open(device, O_RDWR)) < 0)
+
+	fd = open(device, O_RDWR);
+	if (fd < 0)
 		perr("Error: cannot open device: %s", device);
 	check_mount(fd, device);
 	check_safety_of_device_overwrite(fd, device);
@@ -728,7 +723,7 @@ static void check_mount(int fd, const char *device)
 		if (strncmp(strtok(line, " "), device, strlen(device)) == 0) {
 			fclose(fp);
 			close(fd);
-			perr("Error: %s is currently mounted. "
+			perr("Error: %s is currently mounted. " \
 			     "You cannot make a filesystem on this device.",
 			     device);
 		}
@@ -736,6 +731,7 @@ static void check_mount(int fd, const char *device)
 	fclose(fp);
 }
 
+#if HAVE_LIBBLKID
 static void check_safety_of_device_overwrite(int fd, const char *device)
 {
 	int c, c_next;
@@ -769,18 +765,18 @@ static void check_safety_of_device_overwrite(int fd, const char *device)
 
 			if (!blkid_probe_lookup_value(pr, "TYPE",
 							&type, NULL)) {
-				pinfo("WARNING: Device %s appears to contain "
-					"an existing %s superblock.",
-					device, type);
+				pinfo("WARNING: Device %s appears to contain" \
+				      " an existing %s superblock.",
+				      device, type);
 			} else if (!blkid_probe_lookup_value(pr, "PTTYPE",
 								&type, NULL)) {
-				pinfo("WARNING: Device %s appears to contain "
-					"an partition table (%s).",
-					device, type);
+				pinfo("WARNING: Device %s appears to contain" \
+				      " an partition table (%s).",
+				      device, type);
 			} else {
 				if (quiet == 0) {
-					pinfo("Device %s appears to contain "
-						"something weird.", device);
+					pinfo("Device %s appears to contain" \
+					      " something weird.", device);
 				}
 				goto end_check;
 			}
@@ -825,6 +821,7 @@ abort_format:
 	perr("Abort format of device %s", device);
 	return;
 }
+#endif /* HAVE_LIBBLKID */
 
 static void destroy_disk_buffer(void)
 {
@@ -856,11 +853,11 @@ static void init_disk_buffer(long max_blocks)
 static void *map_disk_buffer(blocknr_t blocknr, int clear_flag)
 {
 	if (blocknr >= disk_buffer_size)
-		perr("Internal error: illegal disk buffer access "
+		perr("Internal error: illegal disk buffer access " \
 		     "(blocknr=%llu)", blocknr);
 
 	if (!disk_buffer[blocknr]) {
-		if (posix_memalign(&disk_buffer[blocknr], blocksize, 
+		if (posix_memalign(&disk_buffer[blocknr], blocksize,
 				   blocksize) != 0)
 			cannot_allocate_memory();
 		if (clear_flag)
@@ -871,7 +868,7 @@ static void *map_disk_buffer(blocknr_t blocknr, int clear_flag)
 
 static void read_disk_header(int fd, const char *device)
 {
-	int i, hdr_blocks = ROUNDUP_DIV(NILFS_SB_OFFSET_BYTES, blocksize);
+	int i, hdr_blocks = DIV_ROUND_UP(NILFS_SB_OFFSET_BYTES, blocksize);
 
 	lseek(fd, 0, SEEK_SET);
 	for (i = 0; i < hdr_blocks; i++) {
@@ -942,7 +939,7 @@ static int erase_disk(int fd, struct nilfs_disk_info *di)
 		ret = nilfs_mkfs_discard_range(fd, start, end - start);
 		if (!ret && nilfs_mkfs_discard_zeroes_data(fd)) {
 			if (verbose)
-				pinfo("Discard succeeded and will return 0s "
+				pinfo("Discard succeeded and will return 0s " \
 				      " - skip wiping");
 			goto out;
 		}
@@ -1049,14 +1046,15 @@ static inline void check_ctime(time_t ctime)
 		char cbuf[26], *cbufp;
 
 		ctime_r(&ctime, cbuf);
-		if ((cbufp = rindex(cbuf, '\n')) != NULL)
+		cbufp = rindex(cbuf, '\n');
+		if (cbufp)
 			*cbufp = '\0';
 		pinfo("Warning: Future time: %s (%ld)",
 		      cbuf, (long)ctime);
 	}
 }
 
-const static __u64 ok_features[NILFS_MAX_FEATURE_TYPES] = {
+static const __u64 ok_features[NILFS_MAX_FEATURE_TYPES] = {
 	/* Compat */
 	0,
 	/* Read-only compat */
@@ -1132,8 +1130,7 @@ static void parse_options(int argc, char *argv[])
 	if (quiet)
 		verbose = 0;
 
-	if ((optind == argc) && !show_version_only)
-	{
+	if ((optind == argc) && !show_version_only) {
 		printf("%d %d\n", optind, argc);
 		usage();
 		exit(EXIT_FAILURE);
@@ -1150,11 +1147,11 @@ static void parse_options(int argc, char *argv[])
 	if (fs_features)
 		nilfs_mkfs_edit_feature(fs_features);
 
-        if (argc > 0) {
-                char *cp = strrchr(argv[0], '/');
+	if (argc > 0) {
+		char *cp = strrchr(argv[0], '/');
 
-                progname = (cp ? cp + 1 : argv[0]);
-        }
+		progname = (cp ? cp + 1 : argv[0]);
+	}
 }
 
 /*
@@ -1163,9 +1160,10 @@ static void parse_options(int argc, char *argv[])
 static void usage(void)
 {
 	fprintf(stderr,
-		"Usage: %s [-b block-size] [-B blocks-per-segment] [-c] [-f] \n"
-		"       [-L volume-label] [-m reserved-segments-percentage] \n"
-		"       [-O feature[,...]] \n"
+		"Usage: %s [-b block-size] [-B blocks-per-segment] [-c] " \
+		"[-f]\n" \
+		"       [-L volume-label] [-m reserved-segments-percentage]\n" \
+		"       [-O feature[,...]]\n" \
 		"       [-hnqvKV] device\n",
 		progname);
 }
@@ -1208,7 +1206,7 @@ static void cannot_allocate_memory(void)
 	perr("Error: memory allocation failure");
 }
 
-static void too_small_segment(unsigned long blocks_per_segment, 
+static void too_small_segment(unsigned long blocks_per_segment,
 			      unsigned long required_blocks)
 {
 	perr("Error: too small segment.\n"
@@ -1243,7 +1241,7 @@ static void init_inode(ino_t ino, unsigned type, int mode, unsigned size)
 	raw_inode->i_mode =		cpu_to_le16((type << 12) | mode);
 	raw_inode->i_flags =		0;
 	raw_inode->i_size =		cpu_to_le64(size);
-	raw_inode->i_blocks = 		cpu_to_le64(nilfs.files[ino]->nblocks);
+	raw_inode->i_blocks =		cpu_to_le64(nilfs.files[ino]->nblocks);
 	raw_inode->i_links_count =	cpu_to_le16(1);
 	raw_inode->i_ctime =		cpu_to_le64(nilfs.diskinfo->ctime);
 	raw_inode->i_mtime =		cpu_to_le64(nilfs.diskinfo->ctime);
@@ -1412,7 +1410,7 @@ static void prepare_ifile(void)
 	ino_t ino = 0;
 
 	prepare_blockgrouped_file(blocknr);
-	for (entry_block = blocknr + group_desc_blocks_per_group + 
+	for (entry_block = blocknr + group_desc_blocks_per_group +
 		     bitmap_blocks_per_group;
 	     entry_block < blocknr + fi->nblocks; entry_block++) {
 		raw_inode = map_disk_buffer(entry_block, 1);
@@ -1509,7 +1507,7 @@ static void prepare_sufile(void)
 	struct nilfs_segment_usage *su;
 	unsigned long segnum = 0;
 	int i;
-	
+
 	header = map_disk_buffer(blocknr, 1);
 	header->sh_ncleansegs = cpu_to_le64(nilfs.diskinfo->nsegments -
 					    nr_initial_segments);
@@ -1602,7 +1600,7 @@ static blocknr_t assign_vblocknr(blocknr_t blocknr)
 	entry->de_blocknr = cpu_to_le64(blocknr);
 	entry->de_start = cpu_to_le64(nilfs.cno);
 	entry->de_end = cpu_to_le64(NILFS_CNO_MAX);
-	
+
 	return vblocknr;
 }
 
@@ -1702,10 +1700,10 @@ static void commit_segment(void)
 	for (i = 0, fi = si->files; i < si->nfiles; i++, fi++)
 		update_blocknr(fi, &sum_offset);
 
-	//commit_ifile();
+	/* commit_ifile(); */
 	commit_cpfile();
 	commit_sufile();
-	//commit_dat();
+	/* commit_dat(); */
 
 	fill_in_checksums(si, di->crc_seed);
 
@@ -1771,7 +1769,7 @@ static void prepare_super_block(struct nilfs_disk_info *di)
 	memcpy(raw_sb->s_volume_name, volume_label, sizeof(volume_label));
 }
 
-static void commit_super_block(struct nilfs_disk_info *di, 
+static void commit_super_block(struct nilfs_disk_info *di,
 			       const struct nilfs_segment_ref *segref)
 {
 	__u32 sbsum;
