@@ -60,6 +60,7 @@
 #include <errno.h>
 #include <signal.h>
 #include "nilfs.h"
+#include "util.h"
 #include "vector.h"
 #include "nilfs_gc.h"
 #include "cnoconv.h"
@@ -68,7 +69,7 @@ extern int check_mount(const char *device);
 
 #ifdef _GNU_SOURCE
 #include <getopt.h>
-const static struct option long_option[] = {
+static const struct option long_option[] = {
 	{"help", no_argument, NULL, 'h'},
 	{"verbose", no_argument, NULL, 'v'},
 	{"yes", no_argument, NULL, 'y'},
@@ -91,37 +92,19 @@ const static struct option long_option[] = {
 
 /* get device size through ioctl */
 #ifndef BLKGETSIZE64
-#define BLKGETSIZE64 	_IOR(0x12, 114, size_t)
+#define BLKGETSIZE64	_IOR(0x12, 114, size_t)
 #endif
 
 #ifndef FIFREEZE
-#define FIFREEZE 	_IOWR('X', 119, int)
+#define FIFREEZE	_IOWR('X', 119, int)
 #define FITHAW		_IOWR('X', 120, int)
 #endif
 
-/* general macros */
-#ifndef DIV_ROUND_UP
-#define DIV_ROUND_UP(n,m)   (((n) + (m) - 1) / (m))
-#endif
-
-#ifndef ARRAY_SIZE
-#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof((arr)[0]))
-#endif
-
-#ifndef min_t
-#define min_t(type,x,y) \
-	({ type __x = (x); type __y = (y); __x < __y ? __x : __y; })
-#define max_t(type,x,y) \
-	({ type __x = (x); type __y = (y); __x > __y ? __x : __y; })
-#endif
-
-#define nilfs_cnt64_ge(a, b)	((__s64)(a) - (__s64)(b) >= 0)
-
 /* options */
 static char *progname;
-static int show_version_only = 0;
-static int verbose = 0;
-static int assume_yes = 0;
+static int show_version_only;
+static int verbose;
+static int assume_yes;
 static int show_progress = 1;
 
 /* global variables */
@@ -157,8 +140,8 @@ static int pm_labelwidth = 10;
 static __u64 pm_max;
 static __u64 pm_done;
 static int pm_curpos;
-static int pm_in_progress = 0;  /* 0: off, 1: on, -1: interrupted */
-const static char *pm_label;
+static int pm_in_progress;	/* 0: off, 1: on, -1: interrupted */
+static const char *pm_label;
 
 static void nilfs_resize_logger(int priority, const char *fmt, ...)
 {
@@ -314,7 +297,7 @@ static int nilfs_resize_segment_is_protected(struct nilfs *nilfs, __u64 segnum)
 		return -1;
 	}
 	segseq = nilfs_get_segment_seqnum(nilfs, segment, segnum);
-	if (nilfs_cnt64_ge(segseq, protseq))
+	if (cnt64_ge(segseq, protseq))
 		ret = 1;
 	nilfs_put_segment(nilfs, segment);
 	return ret;
@@ -366,16 +349,16 @@ static int nilfs_resize_check_free_space(struct nilfs *nilfs, __u64 newnsegs)
 		nsegs = (sustat.ss_nsegs - newnsegs) - sustat.ss_ncleansegs
 			+ nrsvsegs;
 		nbytes  = (nsegs * blocks_per_segment) << blocksize_bits;
-		myprintf("Error: the filesystem does not have enough free "
-			 "space.\n"
-			 "       At least %llu more segments (%llu bytes) "
+		myprintf("Error: the filesystem does not have enough free " \
+			 "space.\n" \
+			 "       At least %llu more segments (%llu bytes) " \
 			 "are required.\n", nsegs, nbytes);
 		return -1;
 	} else if (verbose) {
 		nsegs = sustat.ss_ncleansegs - (sustat.ss_nsegs - newnsegs)
 			- nrsvsegs;
 		nbytes  = (nsegs * blocks_per_segment) << blocksize_bits;
-		myprintf("%llu free segments (%llu bytes) will be left "
+		myprintf("%llu free segments (%llu bytes) will be left " \
 			 "after shrinkage.\n", nsegs, nbytes);
 	}
 	return 0;
@@ -406,7 +389,7 @@ nilfs_resize_find_movable_segments(struct nilfs *nilfs, __u64 start,
 		count = min_t(unsigned long, rest, NILFS_RESIZE_NSUINFO);
 		nsi = nilfs_get_suinfo(nilfs, segnum, suinfo, count);
 		if (nsi < 0) {
-			myprintf("Error: operation failed during searching "
+			myprintf("Error: operation failed during searching " \
 				 "movable segments: %s\n", strerror(errno));
 			return -1;
 		}
@@ -422,7 +405,7 @@ nilfs_resize_find_movable_segments(struct nilfs *nilfs, __u64 start,
 			rest--;
 		}
 	}
-	return (snp - segnumv); /* return the number of found segments */
+	return snp - segnumv; /* return the number of found segments */
 }
 
 #if 0
@@ -443,7 +426,7 @@ nilfs_resize_get_latest_segment(struct nilfs *nilfs, __u64 start, __u64 end,
 			      NILFS_RESIZE_NSUINFO);
 		nsi = nilfs_get_suinfo(nilfs, segnum, suinfo, count);
 		if (nsi < 0) {
-			myprintf("Error: operation failed during searching "
+			myprintf("Error: operation failed during searching " \
 				 "latest segment: %s\n", strerror(errno));
 			return -1;
 		}
@@ -481,7 +464,7 @@ nilfs_resize_find_active_segments(struct nilfs *nilfs, __u64 start, __u64 end,
 		count = min_t(unsigned long, rest, NILFS_RESIZE_NSUINFO);
 		nsi = nilfs_get_suinfo(nilfs, segnum, suinfo, count);
 		if (nsi < 0) {
-			myprintf("Error: operation failed during searching "
+			myprintf("Error: operation failed during searching " \
 				 "active segments: %s\n", strerror(errno));
 			return -1;
 		}
@@ -493,7 +476,7 @@ nilfs_resize_find_active_segments(struct nilfs *nilfs, __u64 start, __u64 end,
 			}
 		}
 	}
-	return (snp - segnumv); /* return the number of found segments */
+	return snp - segnumv; /* return the number of found segments */
 }
 
 static ssize_t
@@ -512,7 +495,7 @@ nilfs_resize_find_inuse_segments(struct nilfs *nilfs, __u64 start, __u64 end,
 		count = min_t(unsigned long, rest, NILFS_RESIZE_NSUINFO);
 		nsi = nilfs_get_suinfo(nilfs, segnum, suinfo, count);
 		if (nsi < 0) {
-			myprintf("Error: operation failed during searching "
+			myprintf("Error: operation failed during searching " \
 				 "in-use segments: %s\n", strerror(errno));
 			return -1;
 		}
@@ -523,7 +506,7 @@ nilfs_resize_find_inuse_segments(struct nilfs *nilfs, __u64 start, __u64 end,
 			}
 		}
 	}
-	return (snp - segnumv); /* return the number of found segments */
+	return snp - segnumv; /* return the number of found segments */
 }
 
 static ssize_t
@@ -542,7 +525,7 @@ nilfs_resize_count_inuse_segments(struct nilfs *nilfs, __u64 start, __u64 end)
 		count = min_t(unsigned long, rest, NILFS_RESIZE_NSUINFO);
 		nsi = nilfs_get_suinfo(nilfs, segnum, suinfo, count);
 		if (nsi < 0) {
-			myprintf("Error: operation failed during counting "
+			myprintf("Error: operation failed during counting " \
 				 "in-use segments: %s\n", strerror(errno));
 			return -1;
 		}
@@ -721,7 +704,7 @@ static int nilfs_resize_move_out_active_segments(struct nilfs *nilfs,
 						 unsigned long long start,
 						 unsigned long long end)
 {
-	const static struct timespec retry_interval = { 0, 500000000 };
+	static const struct timespec retry_interval = { 0, 500000000 };
 							/* 500 msec */
 	ssize_t nfound;
 	int retrycnt = 0;
@@ -738,12 +721,12 @@ static int nilfs_resize_move_out_active_segments(struct nilfs *nilfs,
 			break;
 
 		if (retrycnt >= 6) {
-			myprintf("Error: Failed to move active segments -- "
+			myprintf("Error: Failed to move active segments -- " \
 				 "give up.\n");
 			return -1;
 		}
 		if (verbose && !retrycnt) {
-			myprintf("Active segments are found in the range.\n"
+			myprintf("Active segments are found in the range.\n" \
 				 "Trying to move them.\n");
 		}
 		if (nilfs_resize_reclaim_nibble(
@@ -765,7 +748,7 @@ static int nilfs_resize_reclaim_range(struct nilfs *nilfs, __u64 newnsegs)
 		return -1;
 
 	if (newnsegs > sustat.ss_nsegs) {
-		myprintf("Error: Confused. Number of segments became larger "
+		myprintf("Error: Confused. Number of segments became larger " \
 			 "than requested size.\n");
 		return -1;
 	}
@@ -811,7 +794,7 @@ static int nilfs_resize_reclaim_range(struct nilfs *nilfs, __u64 newnsegs)
 		nmoved = nilfs_resize_move_segments(
 			nilfs, segnums, nfound, &reason);
 		if (nmoved < 0) {
-			myprintf("Error: operation failed during moving "
+			myprintf("Error: operation failed during moving " \
 				 "in-use segments: %s\n", strerror(errno));
 			goto out;
 		}
@@ -840,9 +823,8 @@ static void nilfs_print_resize_error(int err, int shrink)
 {
 	myprintf("Error: failed to %s the filesystem: %s\n",
 		 shrink ? "shrink" : "extend", strerror(err));
-	if (err == ENOTTY) {
+	if (err == ENOTTY)
 		myprintf("       This kernel does not support the resize API.\n");
-	}
 }
 
 static int nilfs_resize_prompt(unsigned long long newsize)
@@ -872,7 +854,7 @@ static int nilfs_shrink_online(struct nilfs *nilfs, const char *device,
 	newsb2off = NILFS_SB2_OFFSET_BYTES(newsize);
 	newnsegs = (newsb2off >> blocksize_bits) / blocks_per_segment;
 
-	myprintf("Partition size = %llu bytes.\n"
+	myprintf("Partition size = %llu bytes.\n" \
 		 "Shrink the filesystem size from %llu bytes to %llu bytes.\n",
 		 devsize, fs_devsize, newsize);
 
@@ -880,15 +862,15 @@ static int nilfs_shrink_online(struct nilfs *nilfs, const char *device,
 		goto out;
 
 	if (newnsegs < sustat.ss_nsegs) {
-		myprintf("%llu segments will be truncated from "
+		myprintf("%llu segments will be truncated from " \
 			 "segnum %llu.\n",
 			 (unsigned long long)sustat.ss_nsegs - newnsegs,
 			 (unsigned long long)newnsegs);
 	} else if (newnsegs == sustat.ss_nsegs) {
 		myprintf("No segments will be truncated.\n");
 	} else {
-		myprintf("Error: Confused. Number of segments "
-			 "became larger than requested size:\n"
+		myprintf("Error: Confused. Number of segments " \
+			 "became larger than requested size:\n" \
 			 "  old-nsegs=%llu new-nsegs=%llu\n",
 			 (unsigned long long)sustat.ss_nsegs,
 			 (unsigned long long)newnsegs);
@@ -902,7 +884,7 @@ static int nilfs_shrink_online(struct nilfs *nilfs, const char *device,
 		myprintf("Error: failed to limit allocation range: %s\n",
 			 strerror(errno));
 		if (errno == ENOTTY) {
-			myprintf("       This kernel does not support the "
+			myprintf("       This kernel does not support the " \
 				 "set allocation range API.\n");
 		}
 		goto out;
@@ -931,9 +913,8 @@ static int nilfs_shrink_online(struct nilfs *nilfs, const char *device,
 		if (nilfs_resize_lock_cleaner(nilfs, &sigset) < 0)
 			goto restore_alloc_range;
 
-		if (verbose) {
+		if (verbose)
 			myprintf("Truncating segments.\n");
-		}
 
 		ret = nilfs_resize(nilfs, newsize);
 		err = errno;
@@ -955,7 +936,7 @@ static int nilfs_shrink_online(struct nilfs *nilfs, const char *device,
 			goto restore_alloc_range;
 
 		if (verbose) {
-			myprintf("In-use segment found again. "
+			myprintf("In-use segment found again. " \
 				 "will retry moving them.. (retry = %d)\n",
 				 retry + 1);
 		}
@@ -976,8 +957,8 @@ static int nilfs_extend_online(struct nilfs *nilfs, const char *device,
 	int status = EXIT_FAILURE;
 	sigset_t sigset;
 
-	myprintf("Partition size = %llu bytes.\n"
-		 "Extend the filesystem size from %llu bytes to %llu "
+	myprintf("Partition size = %llu bytes.\n" \
+		 "Extend the filesystem size from %llu bytes to %llu " \
 		 "bytes.\n", devsize, fs_devsize, newsize);
 	if (!assume_yes && nilfs_resize_prompt(newsize) < 0)
 		goto out;
@@ -1017,7 +998,7 @@ static int nilfs_resize_online(const char *device, unsigned long long newsize)
 	nilfs_resize_update_super(sb);
 
 	if (newsize == fs_devsize) {
-		myprintf("No need to resize the filesystem on %s.\n"
+		myprintf("No need to resize the filesystem on %s.\n" \
 			 "It already fits the device.\n", device);
 		status = EXIT_SUCCESS;
 		goto out_unlock;
@@ -1092,12 +1073,24 @@ static int nilfs_resize_parse_size(const char *arg, unsigned long long *sizep)
 		;
 	} else if (endptr[1] == '\0') {
 		switch (endptr[0]) {
-		case 's': size <<= 9; break;
-		case 'K': size <<= 10; break;
-		case 'M': size <<= 20; break;
-		case 'G': size <<= 30; break;
-		case 'T': size <<= 40; break;
-		case 'P': size <<= 50; break;
+		case 's':
+			size <<= 9;
+			break;
+		case 'K':
+			size <<= 10;
+			break;
+		case 'M':
+			size <<= 20;
+			break;
+		case 'G':
+			size <<= 30;
+			break;
+		case 'T':
+			size <<= 40;
+			break;
+		case 'P':
+			size <<= 50;
+			break;
 		default:
 			return -1;
 		}
@@ -1132,15 +1125,14 @@ out:
 
 int main(int argc, char *argv[])
 {
+	char *last;
 	unsigned long long size;
 	struct stat statbuf;
 	char *device;
 	int status, ret;
 
-	if ((progname = strrchr(argv[0], '/')) != NULL)
-		progname++;
-	else
-		progname = argv[0];
+	last = strrchr(argv[0], '/');
+	progname = last ? last + 1 : argv[0];
 
 	nilfs_resize_parse_options(argc, argv);
 	if (show_version_only) {
@@ -1179,7 +1171,7 @@ int main(int argc, char *argv[])
 			goto out;
 		}
 		if (size > devsize) {
-			myprintf("Error: size larger than partition "
+			myprintf("Error: size larger than partition " \
 				 "size (%llu bytes).\n", devsize);
 			goto out;
 		}
@@ -1194,7 +1186,7 @@ int main(int argc, char *argv[])
 			unsigned long long size2;
 
 			size2 = size & ~(sector_size - 1);
-			myprintf("size %llu is not alinged to sector "
+			myprintf("size %llu is not alinged to sector " \
 				 "size. truncated to %llu.\n",
 				 size, size2);
 			size = size2;
@@ -1205,8 +1197,8 @@ int main(int argc, char *argv[])
 
 
 	if (check_mount(device) == 0) {
-		myprintf("Error: %s is not currently mounted. Offline resising"
-			 "\n"
+		myprintf("Error: %s is not currently mounted. Offline " \
+			 "resizing\n" \
 			 "       is not supported at present.\n", device);
 		goto out;
 	} else {
